package generator

import "git.tebibyte.media/fspl/fspl/llvm"
import "git.tebibyte.media/fspl/fspl/entity"

func (this *generator) generateConstantDeclaration (
	ty           *entity.Typedef,
	constantName string,
) (
	*llvm.Global,
	error,
) {
	key := entity.Key {
		Unit:     ty.Unit(),
		Type:     ty.Name,
		Constant: constantName,
	}
	constant := ty.ConstantMap[constantName]

	irType, err := this.typedef(key.OnlyType())
	if err != nil { return nil, err }

	// TODO: have two maps, constantsVal and constantsLoc. if the constant
	// is multivalue, generate a constant global and put it in constantsLoc.
	// if not, put the value in constantsVal. have generator.constant try
	// both. have single generateConstant method that converts if necessary
	// by either loading or allocating a copy on the stack.
	//
	// Have a ConstantSupport method of Type that returns either:
	// - ConstantSupportLoc  - multivalue, stored
	// - ConstantSupportVal  - singlevalue, not stored
	// - ConstantSupportNone - cannot be used as a constant
	//
	// Pointers and anything with a pointer inside it must return
	// ConstantSupportNone. Aggregate types (arrays, structs) must return
	// ConstantSupportLoc if supported at all. Single value types must
	// return ConstantSupportNone if supported at all. This should be
	// checked recursively.
	
	irGlobal := this.module.NewGlobal(key.LinkName(), irType)
	irGlobal.Constant = true

	if ty.Unit() == this.tree.Unit() {
		// in root unit, so generate data
		initial, err := this.generateEvaluated(constant.Value)
		if err != nil { return nil, err }
		irGlobal.Initial  = initial
	}
	
	this.constants[key] = irGlobal
	return irGlobal, nil
}
